@@ -1,8 +1,8 @@
 /*************************************************************************
 ** DVIToSVGActions.cpp                                                  **
 **                                                                      **
-** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
+** This file is part of dvisvgm -- a fast DVI to SVG converter          **
+** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,38 +18,20 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
 #include <cstring>
 #include <ctime>
-#include "BoundingBox.h"
-#include "DVIToSVG.h"
-#include "DVIToSVGActions.h"
-#include "Font.h"
-#include "FontManager.h"
-#include "GlyphTracerMessages.h"
-#include "SpecialManager.h"
-#include "System.h"
-#include "XMLNode.h"
-#include "XMLString.h"
-
+#include "BoundingBox.hpp"
+#include "DVIToSVG.hpp"
+#include "DVIToSVGActions.hpp"
+#include "Font.hpp"
+#include "FontManager.hpp"
+#include "GlyphTracerMessages.hpp"
+#include "System.hpp"
+#include "utility.hpp"
 
 using namespace std;
 
 
-DVIToSVGActions::DVIToSVGActions (DVIToSVG &dvisvg, SVGTree &svg)
-	: _svg(svg), _dvireader(&dvisvg), _pageMatrix(0), _bgcolor(Color::TRANSPARENT), _boxes(0)
-{
-	_currentFontNum = -1;
-	_pageCount = 0;
-}
-
-
-DVIToSVGActions::~DVIToSVGActions () {
-	delete _pageMatrix;
-	delete _boxes;
-}
-
-
 void DVIToSVGActions::reset() {
 	_usedChars.clear();
 	_usedFonts.clear();
@@ -59,28 +41,39 @@ void DVIToSVGActions::reset() {
 }
 
 
-void DVIToSVGActions::setPageMatrix (const Matrix &matrix) {
-	delete _pageMatrix;
-	_pageMatrix = new Matrix(matrix);
+/** Change current horizontal position.
+ *  @param[in] x new horizontal position
+ *  @param[in] forceSVGMove if true, creates an explicit position change in the SVG tree */
+void DVIToSVGActions::moveToX (double x, bool forceSVGMove) {
+	SpecialManager::instance().notifyPositionChange(getX(), getY(), *this);
+	if (forceSVGMove)
+		_svg.setX(x);
 }
 
 
-void DVIToSVGActions::moveToX (double x) {
-	SpecialManager::instance().notifyPositionChange(getX(), getY());
-	_svg.setX(x);
+/** Change current vertical position.
+ *  @param[in] y new vertical position
+ *  @param[in] forceSVGMove if true, creates an explicit position change in the SVG tree */
+void DVIToSVGActions::moveToY (double y, bool forceSVGMove) {
+	SpecialManager::instance().notifyPositionChange(getX(), getY(), *this);
+	if (forceSVGMove)
+		_svg.setY(y);
 }
 
 
-void DVIToSVGActions::moveToY (double y) {
-	SpecialManager::instance().notifyPositionChange(getX(), getY());
-	_svg.setY(y);
+FilePath DVIToSVGActions::getSVGFilePath (unsigned pageno) const {
+	FilePath path;
+	if (auto dvi2svg = dynamic_cast<DVIToSVG*>(_dvireader))
+		path = dvi2svg->getSVGFilePath(pageno);
+	return path;
 }
 
 
-string DVIToSVGActions::getSVGFilename (unsigned pageno) const {
-	if (DVIToSVG *dvi2svg = dynamic_cast<DVIToSVG*>(_dvireader))
-		return dvi2svg->getSVGFilename(pageno);
-	return "";
+string DVIToSVGActions::getBBoxFormatString () const {
+	string boxstr;
+	if (auto dvi2svg = dynamic_cast<DVIToSVG*>(_dvireader))
+		boxstr = dvi2svg->getUserBBoxString();
+	return boxstr;
 }
 
 
@@ -91,26 +84,28 @@ string DVIToSVGActions::getSVGFilename (unsigned pageno) const {
  *  @param[in] c character code relative to the current font
  *  @param[in] vertical true if we're in vertical mode
  *  @param[in] font font to be used */
-void DVIToSVGActions::setChar (double x, double y, unsigned c, bool vertical, const Font *font) {
+void DVIToSVGActions::setChar (double x, double y, unsigned c, bool vertical, const Font &font) {
+	if (_outputLocked)
+		return;
+
 	// If we use SVG fonts there is no need to record all font name/char/size combinations
 	// because the SVG font mechanism handles this automatically. It's sufficient to
 	// record font names and chars. The various font sizes can be ignored here.
 	// For a given font object, Font::uniqueFont() returns the same unique font object for
 	// all fonts with the same name.
-	_usedChars[SVGTree::USE_FONTS ? font->uniqueFont() : font].insert(c);
+	_usedChars[SVGTree::USE_FONTS ? font.uniqueFont() : &font].insert(c);
 
 	// However, we record all required fonts
-	_usedFonts.insert(font);
-	_svg.appendChar(c, x, y, *font);
+	_usedFonts.insert(&font);
+	_svg.appendChar(c, x, y);
 
 	static string fontname;
-	GlyphTracerMessages callback(fontname != font->name(), false);
-	fontname = font->name();
+	GlyphTracerMessages callback(fontname != font.name(), false);
+	fontname = font.name();
 
-	BoundingBox charbox;
 	GlyphMetrics metrics;
-	font->getGlyphMetrics(c, vertical, metrics);
-	const PhysicalFont* pf = dynamic_cast<const PhysicalFont*>(font);
+	font.getGlyphMetrics(c, vertical, metrics);
+	auto pf = dynamic_cast<const PhysicalFont*>(&font);
 	if (PhysicalFont::EXACT_BBOX && pf) {
 		GlyphMetrics exact_metrics;
 		pf->getExactGlyphBox(c, exact_metrics, vertical, &callback);
@@ -127,7 +122,7 @@ void DVIToSVGActions::setChar (double x, double y, unsigned c, bool vertical, co
 		bbox.transform(getMatrix());
 	embed(bbox);
 #if 0
-	XMLElementNode *rect = new XMLElementNode("rect");
+	XMLElement *rect = new XMLElement("rect");
 	rect->addAttribute("x", x-metrics.wl);
 	rect->addAttribute("y", y-metrics.h);
 	rect->addAttribute("width", metrics.wl+metrics.wr);
@@ -137,7 +132,7 @@ void DVIToSVGActions::setChar (double x, double y, unsigned c, bool vertical, co
 	rect->addAttribute("stroke-width", "0.5");
 	_svg.appendToPage(rect);
 	if (metrics.d > 0) {
-		XMLElementNode *line = new XMLElementNode("line");
+		XMLElement *line = new XMLElement("line");
 		line->addAttribute("x1", x-metrics.wl);
 		line->addAttribute("y1", y);
 		line->addAttribute("x2", x+metrics.wr);
@@ -147,7 +142,7 @@ void DVIToSVGActions::setChar (double x, double y, unsigned c, bool vertical, co
 		_svg.appendToPage(line);
 	}
 	if (metrics.wl > 0) {
-		XMLElementNode *line = new XMLElementNode("line");
+		XMLElement *line = new XMLElement("line");
 		line->addAttribute("x1", x);
 		line->addAttribute("y1", y-metrics.h);
 		line->addAttribute("x2", x);
@@ -167,17 +162,20 @@ void DVIToSVGActions::setChar (double x, double y, unsigned c, bool vertical, co
  *  @param[in] height length of the vertical edges
  *  @param[in] width length of the horizontal edges */
 void DVIToSVGActions::setRule (double x, double y, double height, double width) {
+	if (_outputLocked)
+		return;
+
 	// (x,y) is the lower left corner of the rectangle
-	XMLElementNode *rect = new XMLElementNode("rect");
+	auto rect = util::make_unique<XMLElement>("rect");
 	rect->addAttribute("x", x);
 	rect->addAttribute("y", y-height);
 	rect->addAttribute("height", height);
 	rect->addAttribute("width", width);
 	if (!getMatrix().isIdentity())
-		rect->addAttribute("transform", getMatrix().getSVG());
+		rect->addAttribute("transform", getMatrix().toSVG());
 	if (getColor() != Color::BLACK)
 		rect->addAttribute("fill", _svg.getColor().svgColorString());
-	_svg.appendToPage(rect);
+	_svg.appendToPage(std::move(rect));
 
 	// update bounding box
 	BoundingBox bb(x, y-height, x+width, y);
@@ -191,7 +189,7 @@ void DVIToSVGActions::setRule (double x, double y, double height, double width)
  *  font must be previously defined.
  *  @param[in] num unique number of the font in the DVI file (not necessarily equal to the DVI font number)
  *  @param[in] font pointer to the font object (always represents a physical font and never a virtual font) */
-void DVIToSVGActions::setFont (int num, const Font *font) {
+void DVIToSVGActions::setFont (int num, const Font &font) {
 	_currentFontNum = num;
 	_svg.setFont(num, font);
 }
@@ -204,9 +202,9 @@ void DVIToSVGActions::setFont (int num, const Font *font) {
 void DVIToSVGActions::special (const string &spc, double dvi2bp, bool preprocessing) {
 	try {
 		if (preprocessing)
-			SpecialManager::instance().preprocess(spc, this);
+			SpecialManager::instance().preprocess(spc, *this);
 		else
-			SpecialManager::instance().process(spc, dvi2bp, this);
+			SpecialManager::instance().process(spc, dvi2bp, *this);
 		// @@ output message in case of unsupported specials?
 	}
 	catch (const SpecialException &e) {
@@ -215,40 +213,32 @@ void DVIToSVGActions::special (const string &spc, double dvi2bp, bool preprocess
 }
 
 
-/** This method is called when the DVI preamble was read
- *  @param[in] cmt preamble comment text. */
-void DVIToSVGActions::preamble (const string &cmt) {
-}
-
-
-void DVIToSVGActions::postamble () {
-}
-
-
 /** This method is called when a "begin of page (bop)" command was found in the DVI file.
  *  @param[in] pageno physical page number
  *  @param[in] c array with 10 components representing \\count0 ... \\count9. c[0] contains the
  *               current (printed) page number (may differ from page count) */
-void DVIToSVGActions::beginPage (unsigned pageno, Int32 *c) {
+void DVIToSVGActions::beginPage (unsigned pageno, const vector<int32_t>&) {
+	SpecialManager::instance().notifyBeginPage(pageno, *this);
 	_svg.newPage(++_pageCount);
 	_bbox = BoundingBox();  // clear bounding box
-	if (_boxes)
-		_boxes->clear();
+	_boxes.clear();
 }
 
 
 /** This method is called when an "end of page (eop)" command was found in the DVI file. */
 void DVIToSVGActions::endPage (unsigned pageno) {
-	_svg.transformPage(_pageMatrix);
+	SpecialManager::instance().notifyEndPage(pageno, *this);
+	Matrix matrix = _dvireader->getPageTransformation();
+	_svg.transformPage(matrix);
 	if (_bgcolor != Color::TRANSPARENT) {
 		// create a rectangle filled with the background color
-		XMLElementNode *r = new XMLElementNode("rect");
-		r->addAttribute("x", _bbox.minX());
-		r->addAttribute("y", _bbox.minY());
-		r->addAttribute("width", _bbox.width());
-		r->addAttribute("height", _bbox.height());
-		r->addAttribute("fill", _bgcolor.svgColorString());
-		_svg.prependToPage(r);
+		auto rect = util::make_unique<XMLElement>("rect");
+		rect->addAttribute("x", _bbox.minX());
+		rect->addAttribute("y", _bbox.minY());
+		rect->addAttribute("width", _bbox.width());
+		rect->addAttribute("height", _bbox.height());
+		rect->addAttribute("fill", _bgcolor.svgColorString());
+		_svg.prependToPage(std::move(rect));
 	}
 }
 
@@ -258,12 +248,10 @@ void DVIToSVGActions::setBgColor (const Color &color) {
 }
 
 
-void DVIToSVGActions::embed(const BoundingBox& bbox) {
+void DVIToSVGActions::embed(const BoundingBox &bbox) {
 	_bbox.embed(bbox);
-	if (_boxes) {
-		FORALL(*_boxes, BoxMap::iterator, it)
-			it->second.embed(bbox);
-	}
+	for (auto &strboxpair : _boxes)
+		strboxpair.second.embed(bbox);
 }
 
 
@@ -272,16 +260,13 @@ void DVIToSVGActions::embed(const DPair& p, double r) {
 		_bbox.embed(p);
 	else
 		_bbox.embed(p, r);
-	if (_boxes)
-		FORALL(*_boxes, BoxMap::iterator, it)
-			it->second.embed(p, r);
+	for (auto &strboxpair : _boxes)
+		strboxpair.second.embed(p, r);
 }
 
 
 BoundingBox& DVIToSVGActions::bbox(const string& name, bool reset) {
-	if (!_boxes)
-		_boxes = new BoxMap;
-	BoundingBox &box = (*_boxes)[name];
+	BoundingBox &box = _boxes[name];
 	if (reset)
 		box = BoundingBox();
 	return box;
@@ -294,8 +279,8 @@ BoundingBox& DVIToSVGActions::bbox(const string& name, bool reset) {
  *  a rotating dash. */
 void DVIToSVGActions::progress (const char *id) {
 	if (PROGRESSBAR_DELAY < 1000) {
-		static double time=0;
-		// slow down updating of the progress indicator to prevent flickering
+		static double time=System::time();
+		// slow down updating the progress indicator to prevent flickering
 		if (System::time() - time > 0.1) {
 			progress(0, 0, id);
 			time = System::time();
@@ -309,8 +294,8 @@ static int digits (int n) {
 	if (n == 0)
 		return 1;
 	if (n > 0)
-		return int(log10(double(n))+1);
-	return int(log10(double(-n))+2);
+		return util::ilog10(n)+1;
+	return util::ilog10(-n)+2;
 }
 
 
@@ -321,7 +306,7 @@ static int digits (int n) {
 void DVIToSVGActions::progress (size_t current, size_t total, const char *id) {
 	static double time=0;
 	static bool draw=false; // show progress indicator?
-	static const char *prev_id=0;
+	static const char *prev_id=nullptr;
 	if (current == 0 && total > 0) {
 		time = System::time();
 		draw = false;
