@@ -1,8 +1,8 @@
 /*************************************************************************
 ** FileSystem.cpp                                                       **
 **                                                                      **
-** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** This file is part of dvisvgm -- a fast DVI to SVG converter          **
+** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -19,11 +19,13 @@
 *************************************************************************/
 
 #include <config.h>
+#include <algorithm>
 #include <cstdlib>
 #include <cstring>
 #include <fstream>
-#include "FileSystem.h"
-#include "macros.h"
+#include "FileSystem.hpp"
+#include "utility.hpp"
+#include "version.hpp"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -32,22 +34,59 @@
 
 using namespace std;
 
-#ifdef __WIN32__
+#ifdef _WIN32
 	#include <direct.h>
-	#include <windows.h>
-  	const char *FileSystem::DEVNULL = "nul";
-  	const char FileSystem::PATHSEP = '\\';
+	#include "windows.hpp"
+	const char *FileSystem::DEVNULL = "nul";
+	const char FileSystem::PATHSEP = '\\';
 	#define unlink _unlink
 #else
 	#include <dirent.h>
 	#include <pwd.h>
-  	#include <sys/stat.h>
-  	#include <sys/types.h>
-  	const char *FileSystem::DEVNULL = "/dev/null";
-  	const char FileSystem::PATHSEP = '/';
+	#include <sys/stat.h>
+	#include <sys/types.h>
+	const char *FileSystem::DEVNULL = "/dev/null";
+	const char FileSystem::PATHSEP = '/';
 #endif
 
 
+FileSystem FileSystem::_fs;
+string FileSystem::TMPDIR;
+const char *FileSystem::TMPSUBDIR = nullptr;
+
+
+/** Private wrapper function for mkdir: creates a single folder.
+ *  @param[in] dir folder name
+ *  @return true on success */
+static bool s_mkdir (const string &dirname) {
+	bool success = true;
+	if (!FileSystem::exists(dirname)) {
+#ifdef _WIN32
+		success = (_mkdir(dirname.c_str()) == 0);
+#else
+		success = (::mkdir(dirname.c_str(), 0775) == 0);
+#endif
+	}
+	return success;
+}
+
+
+static bool inline s_rmdir (const string &dirname) {
+#ifdef _WIN32
+	return (_rmdir(dirname.c_str()) == 0);
+#else
+	return (::rmdir(dirname.c_str()) == 0);
+#endif
+}
+
+
+FileSystem::~FileSystem () {
+	// remove the subdirectory from the system's temp folder (if empty)
+	if (TMPSUBDIR)
+		s_rmdir(tmpdir());
+}
+
+
 bool FileSystem::remove (const string &fname) {
 	return unlink(fname.c_str()) == 0;
 }
@@ -59,8 +98,8 @@ bool FileSystem::remove (const string &fname) {
  *  @param[in] remove_src remove file 'src' if true
  *  @return true on success */
 bool FileSystem::copy (const string &src, const string &dest, bool remove_src) {
-	ifstream ifs(src.c_str(), ios::in|ios::binary);
-	ofstream ofs(dest.c_str(), ios::out|ios::binary);
+	ifstream ifs(src, ios::in|ios::binary);
+	ofstream ofs(dest, ios::out|ios::binary);
 	if (ifs && ofs) {
 		ofs << ifs.rdbuf();
 		if (!ifs.fail() && !ofs.fail() && remove_src) {
@@ -68,7 +107,7 @@ bool FileSystem::copy (const string &src, const string &dest, bool remove_src) {
 			ifs.close();
 			return remove(src);
 		}
-		else 
+		else
 			return !remove_src;
 	}
 	return false;
@@ -80,13 +119,13 @@ bool FileSystem::rename (const string &oldname, const string &newname) {
 }
 
 
-UInt64 FileSystem::filesize (const string &fname) {
-#ifdef __WIN32__
+uint64_t FileSystem::filesize (const string &fname) {
+#ifdef _WIN32
 	// unfortunately, stat doesn't work properly under Windows
 	// so we have to use this freaky code
 	WIN32_FILE_ATTRIBUTE_DATA attr;
 	GetFileAttributesExA(fname.c_str(), GetFileExInfoStandard, &attr);
-	return (static_cast<UInt64>(attr.nFileSizeHigh) << (8*sizeof(attr.nFileSizeLow))) | attr.nFileSizeLow;
+	return (static_cast<uint64_t>(attr.nFileSizeHigh) << (8*sizeof(attr.nFileSizeLow))) | attr.nFileSizeLow;
 #else
 	struct stat attr;
 	return (stat(fname.c_str(), &attr) == 0) ? attr.st_size : 0;
@@ -95,16 +134,14 @@ UInt64 FileSystem::filesize (const string &fname) {
 
 
 string FileSystem::adaptPathSeperators (string path) {
-	for (size_t i=0; i < path.length(); i++)
-		if (path[i] == PATHSEP)
-			path[i] = '/';
+	std::replace(path.begin(), path.end(), PATHSEP, '/');
 	return path;
 }
 
 
 string FileSystem::getcwd () {
 	char buf[1024];
-#ifdef __WIN32__
+#ifdef _WIN32
 	return adaptPathSeperators(_getcwd(buf, 1024));
 #else
 	return ::getcwd(buf, 1024);
@@ -113,15 +150,15 @@ string FileSystem::getcwd () {
 
 
 /** Changes the work directory.
- *  @param[in] path to new work directory
+ *  @param[in] dir path to new work directory
  *  @return true on success */
-bool FileSystem::chdir (const char *dir) {
+bool FileSystem::chdir (const std::string &dirname) {
 	bool success = false;
-	if (dir) {
-#ifdef __WIN32__
-		success = (_chdir(dir) == 0);
+	if (const char *cdirname = dirname.c_str()) {
+#ifdef _WIN32
+		success = (_chdir(cdirname) == 0);
 #else
-		success = (chdir(dir) == 0);
+		success = (::chdir(cdirname) == 0);
 #endif
 	}
 	return success;
@@ -130,7 +167,7 @@ bool FileSystem::chdir (const char *dir) {
 
 /** Returns the user's home directory. */
 const char* FileSystem::userdir () {
-#ifdef __WIN32__
+#ifdef _WIN32
 	const char *drive=getenv("HOMEDRIVE");
 	const char *path=getenv("HOMEPATH");
 	if (drive && path) {
@@ -138,7 +175,7 @@ const char* FileSystem::userdir () {
 		if (!ret.empty())
 			return ret.c_str();
 	}
-	return 0;
+	return nullptr;
 #else
 	const char *dir=getenv("HOME");
 	if (!dir) {
@@ -152,39 +189,38 @@ const char* FileSystem::userdir () {
 }
 
 
-/** Private wrapper function for mkdir: creates a single folder.
- *  @param[in] dir folder name
- *  @return true on success */
-static bool s_mkdir (const char *dir) {
-	bool success = true;
-	if (!FileSystem::exists(dir)) {
-#ifdef __WIN32__
-		success = (_mkdir(dir) == 0);
+/** Returns the path of the temporary folder. */
+string FileSystem::tmpdir () {
+	string ret;
+	if (!TMPDIR.empty())
+		ret = TMPDIR;
+	else {
+#ifdef _WIN32
+		char buf[MAX_PATH];
+		if (GetTempPath(MAX_PATH, buf))
+			ret = adaptPathSeperators(buf);
+		else
+			ret = ".";
 #else
-		success = (mkdir(dir, 0776) == 0);
+		if (const char *path = getenv("TMPDIR"))
+			ret = path;
+		else
+			ret = "/tmp";
 #endif
+		if (ret.back() == '/')
+			ret.pop_back();
+		static bool initialized=false;
+		if (!initialized && ret != ".") {
+			TMPSUBDIR = PROGRAM_NAME;
+			s_mkdir(ret + "/" + TMPSUBDIR);
+			initialized = true;
+		}
+		if (TMPSUBDIR)
+			ret += string("/") + TMPSUBDIR;
 	}
-	return success;
-}
-
-
-static bool inline s_rmdir (const char *dir) {
-#ifdef __WIN32__
-	return (_rmdir(dir) == 0);
-#else
-	return (rmdir(dir) == 0);
-#endif
-}
-
-
-/** Removes leading and trailing whitespace from a string. */
-static string trim (const string &str) {
-	int first=0, last=str.length()-1;
-	while (isspace(str[first]))
-		first++;
-	while (isspace(str[last]))
-		last--;
-	return str.substr(first, last-first+1);
+	if (!ret.empty() && ret.back() != '/')
+		ret += '/';
+	return ret;
 }
 
 
@@ -192,14 +228,14 @@ static string trim (const string &str) {
  *  the parent folders are also created.
  *  @param[in] dir single folder name or path to folder
  *  @return true if folder(s) could be created */
-bool FileSystem::mkdir (const char *dir) {
+bool FileSystem::mkdir (const string &dirname) {
 	bool success = false;
-	if (dir) {
+	if (const char *cdirname = dirname.c_str()) {
 		success = true;
-		const string dirstr = adaptPathSeperators(trim(dir));
+		const string dirstr = adaptPathSeperators(util::trim(cdirname));
 		for (size_t pos=1; success && (pos = dirstr.find('/', pos)) != string::npos; pos++)
-			success &= s_mkdir(dirstr.substr(0, pos).c_str());
-		success &= s_mkdir(dirstr.c_str());
+			success &= s_mkdir(dirstr.substr(0, pos));
+		success &= s_mkdir(dirstr);
 	}
 	return success;
 }
@@ -208,23 +244,23 @@ bool FileSystem::mkdir (const char *dir) {
 /** Removes a directory and its contents.
  *  @param[in] dirname path to directory
  *  @return true on success */
-bool FileSystem::rmdir (const char *dirname) {
+bool FileSystem::rmdir (const string &dirname) {
 	bool ok = false;
-	if (dirname && isDirectory(dirname)) {
+	if (isDirectory(dirname)) {
 		ok = true;
-#ifdef __WIN32__
-		string pattern = string(dirname) + "/*";
+#ifdef _WIN32
+		string pattern = dirname + "/*";
 		WIN32_FIND_DATA data;
 		HANDLE h = FindFirstFile(pattern.c_str(), &data);
 		bool ready = (h == INVALID_HANDLE_VALUE);
 		while (!ready && ok) {
 			const char *fname = data.cFileName;
-			string path = string(dirname) + "/" + fname;
-			if (isDirectory(path.c_str())) {
+			string path = dirname + "/" + fname;
+			if (isDirectory(path)) {
 				if (strcmp(fname, ".") != 0 && strcmp(fname, "..") != 0)
-					ok = rmdir(path.c_str()) && s_rmdir(path.c_str());
+					ok = rmdir(path) && s_rmdir(path);
 			}
-			else if (isFile(path.c_str()))
+			else if (isFile(path))
 				ok = remove(path);
 			else
 				ok = false;
@@ -232,16 +268,16 @@ bool FileSystem::rmdir (const char *dirname) {
 		}
 		FindClose(h);
 #else
-		if (DIR *dir = opendir(dirname)) {
+		if (DIR *dir = opendir(dirname.c_str())) {
 			struct dirent *ent;
 			while ((ent = readdir(dir)) && ok) {
 				const char *fname = ent->d_name;
-				string path = string(fname) + "/" + fname;
-				if (isDirectory(path.c_str())) {
-				  	if (strcmp(fname, ".") != 0 && strcmp(fname, "..") != 0)
-						ok = rmdir(path.c_str()) && s_rmdir(path.c_str());
+				string path = dirname + "/" + fname;
+				if (isDirectory(path)) {
+					if (strcmp(fname, ".") != 0 && strcmp(fname, "..") != 0)
+						ok = rmdir(path) && s_rmdir(path);
 				}
-				else if (isFile(path.c_str()))
+				else if (isFile(path))
 					ok = remove(path);
 				else
 					ok = false;
@@ -256,75 +292,78 @@ bool FileSystem::rmdir (const char *dirname) {
 
 
 /** Checks if a file or directory exits. */
-bool FileSystem::exists (const char *fname) {
-	if (!fname)
-		return false;
-#ifdef __WIN32__
-	return GetFileAttributes(fname) != INVALID_FILE_ATTRIBUTES;
+bool FileSystem::exists (const string &fname) {
+	if (const char *cfname = fname.c_str()) {
+
+#ifdef _WIN32
+		return GetFileAttributes(cfname) != INVALID_FILE_ATTRIBUTES;
 #else
-	struct stat attr;
-	return stat(fname, &attr) == 0;
+		struct stat attr;
+		return stat(cfname, &attr) == 0;
 #endif
+	}
+	return false;
 }
 
 
 /** Returns true if 'fname' references a directory. */
-bool FileSystem::isDirectory (const char *fname) {
-	if (!fname)
-		return false;
-#ifdef __WIN32__
-	return GetFileAttributes(fname) & FILE_ATTRIBUTE_DIRECTORY;
+bool FileSystem::isDirectory (const string &fname) {
+	if (const char *cfname = fname.c_str()) {
+#ifdef _WIN32
+		auto attr = GetFileAttributes(cfname);
+		return attr != INVALID_FILE_ATTRIBUTES && (attr & FILE_ATTRIBUTE_DIRECTORY);
 #else
-	struct stat attr;
-	return stat(fname, &attr) == 0 && S_ISDIR(attr.st_mode);
+		struct stat attr;
+		return stat(cfname, &attr) == 0 && S_ISDIR(attr.st_mode);
 #endif
+	}
+	return false;
 }
 
 
 /** Returns true if 'fname' references a file. */
-bool FileSystem::isFile (const char *fname) {
-	if (!fname)
-		return false;
-#ifdef __WIN32__
-	ifstream ifs(fname);
-	return (bool)ifs;
+bool FileSystem::isFile (const string &fname) {
+	if (const char *cfname = fname.c_str()) {
+#ifdef _WIN32
+		ifstream ifs(cfname);
+		return (bool)ifs;
 #else
-	struct stat attr;
-	return stat(fname, &attr) == 0 && S_ISREG(attr.st_mode);
+		struct stat attr;
+		return stat(cfname, &attr) == 0 && S_ISREG(attr.st_mode);
 #endif
+	}
+	return false;
 }
 
 
-int FileSystem::collect (const char *dirname, vector<string> &entries) {
+int FileSystem::collect (const std::string &dirname, vector<string> &entries) {
 	entries.clear();
-#ifdef __WIN32__
-	string pattern = string(dirname) + "/*";
+#ifdef _WIN32
+	string pattern = dirname + "/*";
 	WIN32_FIND_DATA data;
 	HANDLE h = FindFirstFile(pattern.c_str(), &data);
 	bool ready = (h == INVALID_HANDLE_VALUE);
 	while (!ready) {
 		string fname = data.cFileName;
-		string path = string(dirname)+"/"+fname;
-		string typechar = isFile(path.c_str()) ? "f" : isDirectory(path.c_str()) ? "d" : "?";
+		string path = dirname+"/"+fname;
+		string typechar = isFile(path) ? "f" : isDirectory(path) ? "d" : "?";
 		if (fname != "." && fname != "..")
-			entries.push_back(typechar+fname);
+			entries.emplace_back(typechar+fname);
 		ready = !FindNextFile(h, &data);
 	}
 	FindClose(h);
 #else
-	if (DIR *dir = opendir(dirname)) {
+	if (DIR *dir = opendir(dirname.c_str())) {
 		struct dirent *ent;
 		while ((ent = readdir(dir))) {
 			string fname = ent->d_name;
-			string path = string(dirname)+"/"+fname;
-			string typechar = isFile(path.c_str()) ? "f" : isDirectory(path.c_str()) ? "d" : "?";
+			string path = dirname+"/"+fname;
+			string typechar = isFile(path) ? "f" : isDirectory(path) ? "d" : "?";
 			if (fname != "." && fname != "..")
-				entries.push_back(typechar+fname);
+				entries.emplace_back(typechar+fname);
 		}
 		closedir(dir);
 	}
 #endif
 	return entries.size();
 }
-
-
