@@ -1,8 +1,8 @@
 /*************************************************************************
 ** GFGlyphTracer.cpp                                                    **
 **                                                                      **
-** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** This file is part of dvisvgm -- a fast DVI to SVG converter          **
+** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,25 +18,23 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
-#include "GFGlyphTracer.h"
-#include "Pair.h"
+#include "GFGlyphTracer.hpp"
 
 using namespace std;
 
-GFGlyphTracer::GFGlyphTracer () : GFTracer(_ifs, 0), _glyph(0), _callback(0)
+GFGlyphTracer::GFGlyphTracer () : GFTracer(_ifs, 0)
 {
 }
 
 /** Constructs a new glyph tracer.
  *  @param[in] is GF input stream
- *  @param[in] upp target units per TeX point */
+ *  @param[in] upp target units per PS point */
 GFGlyphTracer::GFGlyphTracer (string &fname, double upp, Callback *cb)
-   : GFTracer(_ifs, upp), _glyph(0), _callback(cb)
+	: GFTracer(_ifs, upp), _callback(cb)
 {
 	if (_callback)
 		_callback->setFont(fname);
-	_ifs.open(fname.c_str(), ios::binary);
+	_ifs.open(fname, ios::binary);
 }
 
 
@@ -46,24 +44,24 @@ void GFGlyphTracer::reset (string &fname, double upp) {
 	if (_ifs.is_open())
 		_ifs.close();
 	unitsPerPoint(upp);
-	_ifs.open(fname.c_str(), ios::binary);
+	_ifs.open(fname, ios::binary);
 }
 
 
-bool GFGlyphTracer::executeChar (UInt8 c) {
-	if (_glyph) {
-		if (_callback)
-			_callback->beginChar(c);
-		bool ok = GFTracer::executeChar(c);
-		if (_callback) {
-			if (ok)
-				_callback->endChar(c);
-			else
-				_callback->emptyChar(c);
-			return ok;
-		}
+bool GFGlyphTracer::executeChar (uint8_t c) {
+	if (!_glyph)
+		return false;
+
+	if (_callback)
+		_callback->beginChar(c);
+	bool ok = GFTracer::executeChar(c);
+	if (_callback) {
+		if (ok)
+			_callback->endChar(c);
+		else
+			_callback->emptyChar(c);
 	}
-	return false;
+	return ok;
 }
 
 
@@ -87,7 +85,7 @@ void GFGlyphTracer::closePath () {
 }
 
 
-void GFGlyphTracer::endChar (UInt32 c) {
-	_glyph->newpath();
+void GFGlyphTracer::endChar (uint32_t c) {
+	_glyph->clear();
 	GFTracer::endChar(c);
 }
