@@ -1,8 +1,8 @@
 /*************************************************************************
 ** GFReader.cpp                                                         **
 **                                                                      **
-** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** This file is part of dvisvgm -- a fast DVI to SVG converter          **
+** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,33 +18,29 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <config.h>
-#include <iostream>
+#include <algorithm>
 #include <sstream>
-#include "GFReader.h"
-#include "macros.h"
-#include "SignalHandler.h"
+#include "FixWord.hpp"
+#include "GFReader.hpp"
+#include "Length.hpp"
+#include "SignalHandler.hpp"
 
 using namespace std;
 
-struct GFCommand
-{
+
+struct GFCommand {
 	void (GFReader::*method)(int);
 	int numBytes;
 };
 
 
-static inline double fix2double (Int32 fix) {
-	return double(fix)/(1 << 20);
-}
-
-
-static inline double scaled2double (Int32 scaled) {
+/** Converts a scaled value to double */
+static inline double scaled2double (int32_t scaled) {
 	return double(scaled)/(1 << 16);
 }
 
 
-GFReader::GFReader (istream &is) : _in(is), _penDown(false)
+GFReader::GFReader (istream &is) : _in(is), _insideCharDef(false), _penDown(false)
 {
 	_minX = _maxX = _minY = _maxY = _x = _y = 0;
 	_currentChar = 0;
@@ -54,33 +50,31 @@ GFReader::GFReader (istream &is) : _in(is), _penDown(false)
 }
 
 
-UInt32 GFReader::readUnsigned (int bytes) {
-	UInt32 ret = 0;
+uint32_t GFReader::readUnsigned (int bytes) {
+	uint32_t ret = 0;
 	for (int i=bytes-1; i >= 0 && !_in.eof(); i--) {
-		UInt32 b = _in.get();
+		uint32_t b = _in.get();
 		ret |= b << (8*i);
 	}
 	return ret;
 }
 
 
-Int32 GFReader::readSigned (int bytes) {
-	Int32 ret = _in.get();
+int32_t GFReader::readSigned (int bytes) {
+	uint32_t ret = _in.get();
 	if (ret & 128)        // negative value?
 		ret |= 0xffffff00;
 	for (int i=bytes-2; i >= 0 && !_in.eof(); i--)
 		ret = (ret << 8) | _in.get();
-	return ret;
+	return int32_t(ret);
 }
 
 
 string GFReader::readString (int bytes) {
-	vector<char> buf(bytes+1);
-	if (bytes > 0)
-		_in.get(&buf[0], bytes+1);  // reads 'bytes' bytes (pos. bytes+1 is set to 0)
-	else
-		buf[0] = 0;
-	return &buf[0];
+	bytes = max(0, bytes);
+	string str(bytes, '\0');
+	_in.read(&str[0], bytes);
+	return str;
 }
 
 
@@ -89,8 +83,8 @@ string GFReader::readString (int bytes) {
  *  @return opcode of the executed command */
 int GFReader::executeCommand () {
 	SignalHandler::instance().check();
-   /* Each cmdFOO command reads the necessary number of bytes from the stream so executeCommand
-   doesn't need to know the exact GF command format. Some cmdFOO methods are used for multiple
+	/* Each cmdFOO command reads the necessary number of bytes from the stream so executeCommand
+	doesn't need to know the exact GF command format. Some cmdFOO methods are used for multiple
 	GF commands because they only differ in the size of their parameters. */
 	static const GFCommand commands[] = {
 		{&GFReader::cmdPaint, 1}, {&GFReader::cmdPaint, 2}, {&GFReader::cmdPaint, 3},                        // 64-66
@@ -112,11 +106,8 @@ int GFReader::executeCommand () {
 		cmdPaint0(opcode);
 	else if (opcode >= 74 && opcode <= 238)
 		cmdNewRow(opcode-74);
-	else if (opcode >= 250) {
-		ostringstream oss;
-		oss << "undefined GF command (opcode " << opcode << ")";
-		throw GFException(oss.str());
-	}
+	else if (opcode >= 250)
+		throw GFException("undefined GF command (opcode " + std::to_string(opcode) + ")");
 	else {
 		int offset = opcode <= 73 ? 64 : 239-(73-64+1);
 		const GFCommand &cmd = commands[opcode-offset];
@@ -127,14 +118,14 @@ int GFReader::executeCommand () {
 }
 
 
-bool GFReader::executeChar (UInt8 c) {
+bool GFReader::executeChar (uint8_t c) {
 	_in.clear();
 	if (_charInfoMap.empty())
 		executePostamble();          // read character info
 	_in.clear();
-	Iterator it = _charInfoMap.find(c);
+	auto it = _charInfoMap.find(c);
 	if (_in && it != _charInfoMap.end()) {
-		_in.seekg(it->second.location, ios_base::beg);
+		_in.seekg(it->second.location);
 		while (executeCommand() != 69);  // execute all commands until eoc is reached
 		return true;
 	}
@@ -156,39 +147,38 @@ bool GFReader::executeAllChars () {
 }
 
 
+bool GFReader::executePreamble () {
+	_in.clear();
+	if (!_in)
+		return false;
+	_in.seekg(0);
+	executeCommand();
+	return true;
+}
+
+
 bool GFReader::executePostamble () {
 	_in.clear();
 	if (!_in)
 		return false;
-	_in.seekg(-1, ios_base::end);
-	while (_in.peek() == 223)         // skip fill bytes
-		_in.seekg(-1, ios_base::cur);
-	_in.seekg(-4, ios_base::cur);
-	UInt32 q = readUnsigned(4);      // pointer to begin of postamble
-	_in.seekg(q, ios_base::beg);      // now on begin of postamble
+	_in.seekg(-1, ios::end);
+	while (_in.peek() == 223)        // skip fill bytes
+		_in.seekg(-1, ios::cur);
+	if (_in.peek() != 131)
+		throw GFException("invalid identification byte in postpost");
+	_in.seekg(-5, ios::cur);         // now on postpost
+	if (_in.get() != 249)
+		throw GFException("invalid GF file");
+	uint32_t q = readUnsigned(4);    // pointer to begin of postamble
+	_in.seekg(q);                    // now on begin of postamble
 	while (executeCommand() != 249); // execute all commands until postpost is reached
 	return true;
 }
 
 
-/** Returns the design size of this font int TeX point units. */
-double GFReader::getDesignSize () const {
-	return fix2double(_designSize);
-}
-
-/** Returns the number of horizontal pixels per point. */
-double GFReader::getHPixelsPerPoint () const {
-	return scaled2double(_hppp);
-}
-
-/** Returns the number of vertical pixels per point. */
-double GFReader::getVPixelsPerPoint () const {
-	return scaled2double(_vppp);
-}
-
-/** Returns the width of character c in TeX point units */
-double GFReader::getCharWidth (UInt32 c) const {
-	ConstIterator it = _charInfoMap.find(c%256);
+/** Returns the width of character c in PS point units */
+double GFReader::getCharWidth (uint32_t c) const {
+	auto it = _charInfoMap.find(c%256);
 	return it == _charInfoMap.end() ? 0 : it->second.width*getDesignSize()/(1<<24);
 }
 
@@ -197,25 +187,25 @@ double GFReader::getCharWidth (UInt32 c) const {
 
 /** Reads the preamble. */
 void GFReader::cmdPre (int) {
-	UInt32 i = readUnsigned(1);
-	if (i == 131) {
-		UInt32 k = readUnsigned(1);
+	uint32_t i = readUnsigned(1);
+	if (i != 131)
+		throw GFException("invalid identification byte in preamble");
+	else {
+		uint32_t k = readUnsigned(1);
 		string s = readString(k);
 		preamble(s);
 	}
-	else
-		throw GFException("invalid identification number in GF preamble");
 }
 
 
 /** Reads the postamble. */
 void GFReader::cmdPost (int) {
-	readUnsigned(4);               // pointer to byte after final eoc
-	_designSize = readUnsigned(4);  // design size of font in points
+	readUnsigned(4);                // pointer to byte after final eoc
+	_designSize = double(FixWord(readUnsigned(4)))*Length::pt2bp; // design size of font in points
 	_checksum   = readUnsigned(4);  // checksum
-	_hppp       = readUnsigned(4);  // horizontal pixels per point (scaled int)
-	_vppp       = readUnsigned(4);  // vertical pixels per point (scaled int)
-	_in.seekg(16, ios_base::cur);   // skip x and y bounds
+	_hppp       = scaled2double(readUnsigned(4))/Length::pt2bp; // horizontal pixels per PS point
+	_vppp       = scaled2double(readUnsigned(4))/Length::pt2bp; // vertical pixels per PS point
+	_in.seekg(16, ios::cur);        // skip x and y bounds
 	postamble();
 }
 
@@ -223,11 +213,10 @@ void GFReader::cmdPost (int) {
 /** Reads trailing bytes at end of stream. */
 void GFReader::cmdPostPost (int) {
 	readUnsigned(4);   // pointer to begin of postamble
-	UInt32 i = readUnsigned(1);
-	if (i == 131)
-		while (readUnsigned(1) == 223); // skip fill bytes
-	else
-		throw GFException("invalid identification number in GF preamble");
+	uint32_t i = readUnsigned(1);
+	if (i != 131)
+		throw GFException("invalid identification byte in postpost");
+	while (readUnsigned(1) == 223); // skip fill bytes
 }
 
 
@@ -235,10 +224,12 @@ void GFReader::cmdPostPost (int) {
  *  and advances the cursor by n.
  *  @param[in] n number of pixels to be inverted */
 void GFReader::cmdPaint0 (int n) {
-	if (_penDown)                    // set pixels?
+	if (!_insideCharDef)
+		throw GFException("character-related command outside BOC and EOC");
+	if (_penDown)  // set pixels?
 		_bitmap.setBits(_y, _x, n);
 	_x += n;
-	_penDown = !_penDown;             // inverse pen state
+	_penDown = !_penDown;  // invert pen state
 }
 
 
@@ -247,7 +238,7 @@ void GFReader::cmdPaint0 (int n) {
  *  the input stream.
  *  @param[in] len size of n in bytes */
 void GFReader::cmdPaint (int len) {
-	UInt32 pixels = readUnsigned(len);
+	uint32_t pixels = readUnsigned(len);
 	cmdPaint0(pixels);
 }
 
@@ -264,6 +255,7 @@ void GFReader::cmdBoc (int) {
 	_y = _maxY;
 	_penDown = false;
 	_bitmap.resize(_minX, _maxX, _minY, _maxY);
+	_insideCharDef = true;
 	beginChar(_currentChar);
 }
 
@@ -271,22 +263,26 @@ void GFReader::cmdBoc (int) {
 /** Beginning of character (compact format). */
 void GFReader::cmdBoc1 (int) {
 	_currentChar = readUnsigned(1);
-	UInt32 dx = readUnsigned(1);
+	uint32_t dx = readUnsigned(1);
 	_maxX = readUnsigned(1);
 	_minX = _maxX - dx;
-	UInt32 dy = readUnsigned(1);
+	uint32_t dy = readUnsigned(1);
 	_maxY = readUnsigned(1);
 	_minY = _maxY - dy;
 	_x = _minX;
 	_y = _maxY;
 	_penDown = false;
 	_bitmap.resize(_minX, _maxX, _minY, _maxY);
+	_insideCharDef = true;
 	beginChar(_currentChar);
 }
 
 
 /** End of character. */
 void GFReader::cmdEoc (int) {
+	if (!_insideCharDef)
+		throw GFException("misplaced EOC");
+	_insideCharDef = false;
 	endChar(_currentChar);
 }
 
@@ -296,6 +292,8 @@ void GFReader::cmdEoc (int) {
  *  @param[in] len if 0: move to next row, otherwise: number of bytes to read.
  *                 The read value denotes the number of rows to be skipped.  */
 void GFReader::cmdSkip (int len) {
+	if (!_insideCharDef)
+		throw GFException("character-related command outside BOC and EOC");
 	if (len == 0)
 		_y--;
 	else
@@ -309,6 +307,8 @@ void GFReader::cmdSkip (int len) {
  *  the paint color to black.
  *  @param[in] col pixel/column number */
 void GFReader::cmdNewRow (int col) {
+	if (!_insideCharDef)
+		throw GFException("character-related command outside BOC and EOC");
 	_x = _minX + col ;
 	_y--;
 	_penDown = true;
@@ -316,14 +316,14 @@ void GFReader::cmdNewRow (int col) {
 
 
 void GFReader::cmdXXX (int len) {
-	UInt32 n = readUnsigned(len);
+	uint32_t n = readUnsigned(len);
 	string str = readString(n);
 	special(str);
 }
 
 
 void GFReader::cmdYYY (int) {
-	Int32 y = readSigned(4);
+	int32_t y = readSigned(4);
 	numspecial(y);
 }
 
@@ -335,23 +335,22 @@ void GFReader::cmdNop (int) {
 
 /** Reads character locator (part of postamble) */
 void GFReader::cmdCharLoc0 (int) {
-	UInt8 c  = readUnsigned(1); // character code mod 256
-	UInt8 dm = readUnsigned(1); //
-	Int32 w  = readSigned(4);   // (1<<24)*characterWidth/designSize
-	Int32 p   = readSigned(4);  // pointer to begin of (last) character data
-	Int32 dx  = 65536*dm;
-	Int32 dy  = 0;
-	_charInfoMap[c] = CharInfo(dx, dy, w, p);
+	uint8_t c  = readUnsigned(1); // character code mod 256
+	uint8_t dm = readUnsigned(1); // = dx/65536
+	int32_t w  = readSigned(4);   // (1<<24)*characterWidth/designSize
+	int32_t p  = readSigned(4);   // pointer to begin of (last) character data
+	int32_t dx = 65536*dm;        // horizontal escapement
+	int32_t dy = 0;               // vertical escapement
+	_charInfoMap.emplace(c, CharInfo(dx, dy, w, p));
 }
 
 
 /** Reads character locator (part of postamble) */
 void GFReader::cmdCharLoc (int) {
-	UInt32 c = readUnsigned(1); // character code mod 256
-	Int32 dx = readSigned(4);   // horizontal escapement (scaled pixel units)
-	Int32 dy = readSigned(4);   // vertical escapement (scaled pixel units)
-	Int32 w  = readSigned(4);   // (1<<24)*characterWidth/designSize
-	Int32 p  = readSigned(4);   // pointer to begin of (last) character data
-	_charInfoMap[c] = CharInfo(dx, dy, w, p);
+	uint8_t c  = readUnsigned(1); // character code mod 256
+	int32_t dx = readSigned(4);   // horizontal escapement (scaled pixel units)
+	int32_t dy = readSigned(4);   // vertical escapement (scaled pixel units)
+	int32_t w  = readSigned(4);   // (1<<24)*characterWidth/designSize
+	int32_t p  = readSigned(4);   // pointer to begin of (last) character data
+	_charInfoMap.emplace(c, CharInfo(dx, dy, w, p));
 }
-
