@@ -66,7 +66,7 @@ static string get_path_from_registry () {
 					if (major_version >= 7) {
 						char dll_path[512];  // path to Ghostscript DLL stored in the registry
 						DWORD length = 512;
-						if (RegGetValueA(hkey, subkey, "GS_DLL", RRF_RT_REG_SZ, 0, dll_path, &length) == ERROR_SUCCESS) {
+						if (RegGetValueA(hkey, subkey, "GS_DLL", RRF_RT_REG_SZ, nullptr, dll_path, &length) == ERROR_SUCCESS) {
 							RegCloseKey(hkey);
 							return dll_path;
 						}
@@ -160,7 +160,6 @@ Ghostscript::Ghostscript ()
 	: DLLoader(get_libgs(LIBGS_NAME))
 #endif
 {
-	_inst = 0;
 }
 
 
@@ -173,7 +172,6 @@ Ghostscript::Ghostscript (int argc, const char **argv, void *caller)
 	: DLLoader(get_libgs(LIBGS_NAME))
 #endif
 {
-	_inst = 0;
 	init(argc, argv, caller);
 }
 
@@ -191,12 +189,12 @@ bool Ghostscript::init (int argc, const char **argv, void *caller) {
 	if (!_inst) {
 		int status = new_instance(&_inst, caller);
 		if (status < 0)
-			_inst = 0;
+			_inst = nullptr;
 		else {
-			init_with_args(argc, (char**)argv);
+			init_with_args(argc, const_cast<char**>(argv));
 		}
 	}
-	return _inst != 0;
+	return _inst != nullptr;
 }
 
 
@@ -257,7 +255,7 @@ int Ghostscript::new_instance (void **psinst, void *caller) {
 #else
 	if (auto fn = LOAD_SYMBOL(gsapi_new_instance))
 		return fn(psinst, caller);
-	*psinst = 0;
+	*psinst = nullptr;
 	return 0;
 #endif
 }
