@@ -2,7 +2,7 @@
 ** PSInterpreter.cpp                                                    **
 **                                                                      **
 ** This file is part of dvisvgm -- a fast DVI to SVG converter          **
-** Copyright (C) 2005-2018 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -33,29 +33,34 @@
 using namespace std;
 
 
-const char *PSInterpreter::GSARGS[] = {
-	"gs",                // dummy name
-	"-q",                // be quiet, suppress gs banner
-	"-dSAFER",           // disallow writing of files
-	"-dNODISPLAY",       // we don't need a display device
-	"-dNOPAUSE",         // keep going
-	"-dWRITESYSTEMDICT", // leave systemdict writable as some operators must be replaced
-	"-dNOPROMPT",
-//	"-dNOBIND",
-};
-
-
 /** Constructs a new PSInterpreter object.
  *  @param[in] actions template methods to be executed after recognizing the corresponding PS operator. */
 PSInterpreter::PSInterpreter (PSActions *actions)
-	: _mode(PS_NONE), _actions(actions), _filter(0), _bytesToRead(0), _inError(false), _initialized(false)
+	: _mode(PS_NONE), _actions(actions)
 {
 }
 
 
 void PSInterpreter::init () {
 	if (!_initialized) {
-		_gs.init(sizeof(GSARGS)/sizeof(char*), GSARGS, this);
+		vector<const char*> gsargs {
+			"gs",                // dummy name
+			"-q",                // be quiet, suppress gs banner
+			"-dNODISPLAY",       // we don't need a display device
+			"-dNOPAUSE",         // keep going
+			"-dWRITESYSTEMDICT", // leave systemdict writable as some operators must be replaced
+			"-dNOPROMPT"
+		};
+		if (int gsrev = _gs.revision()) {
+			gsargs.emplace_back(gsrev == 922 ? "-dREALLYDELAYBIND" : "-dDELAYBIND");
+			// As of GS 9.50, -dSAFER is active by default which leads to warnings
+			// in conjunction with -dDELAYBIND and -dWRITESYSTEMDICT.
+			// Thus, -dDELAYSAFER (or -dNOSAFER) must be added.
+			// https://www.ghostscript.com/doc/9.50/Use.htm#Safer
+			if (gsrev >= 950)
+				gsargs.emplace_back("-dDELAYSAFER");
+		}
+		_gs.init(gsargs.size(), gsargs.data(), this);
 		_gs.set_stdio(input, output, error);
 		_initialized = true;
 		// Before executing any random PS code redefine some operators and run
@@ -113,7 +118,7 @@ bool PSInterpreter::execute (const char *str, size_t len, bool flush) {
 
 	if (_filter && _filter->active()) {
 		PSFilter *filter = _filter;
-		_filter = 0;             // prevent recursion when filter calls execute()
+		_filter = nullptr;       // prevent recursion when filter calls execute()
 		filter->execute(str, len);
 		if (filter->active())    // filter still active after execution?
 			_filter = filter;
@@ -191,7 +196,7 @@ int GSDLLCALL PSInterpreter::input (void *inst, char *buf, int len) {
  *  @param[in] len number of characters in buf
  *  @return number of processed characters (equals 'len') */
 int GSDLLCALL PSInterpreter::output (void *inst, const char *buf, int len) {
-	PSInterpreter *self = static_cast<PSInterpreter*>(inst);
+	auto self = static_cast<PSInterpreter*>(inst);
 	if (self && self->_actions) {
 		const size_t MAXLEN = 512;    // maximal line length (longer lines are of no interest)
 		const char *end = buf+len-1;  // last position of buf
@@ -206,7 +211,7 @@ int GSDLLCALL PSInterpreter::output (void *inst, const char *buf, int len) {
 			vector<char> &linebuf = self->_linebuf;  // just a shorter name...
 			if ((*last == '\n' || !self->active()) || self->_inError) {
 				if (linelength + linebuf.size() > 1) {  // prefix "dvi." plus final newline
-					SplittedCharInputBuffer ib(linebuf.empty() ? 0 : &linebuf[0], linebuf.size(), first, linelength);
+					SplittedCharInputBuffer ib(linebuf.empty() ? nullptr : &linebuf[0], linebuf.size(), first, linelength);
 					BufferInputReader in(ib);
 					if (self->_inError)
 						self->_errorMessage += string(first, linelength);
@@ -215,7 +220,7 @@ int GSDLLCALL PSInterpreter::output (void *inst, const char *buf, int len) {
 						if (in.check("Unrecoverable error: ")) {
 							self->_errorMessage.clear();
 							while (!in.eof())
-								self->_errorMessage += in.get();
+								self->_errorMessage += char(in.get());
 							self->_inError = true;
 						}
 						else if (in.check("dvi."))
@@ -271,6 +276,7 @@ void PSInterpreter::callActions (InputReader &in) {
 		{"rotate",         { 1, &PSActions::rotate}},
 		{"save",           { 1, &PSActions::save}},
 		{"scale",          { 2, &PSActions::scale}},
+		{"setblendmode",   { 1, &PSActions::setblendmode}},
 		{"setcmykcolor",   { 4, &PSActions::setcmykcolor}},
 		{"setdash",        {-1, &PSActions::setdash}},
 		{"setgray",        { 1, &PSActions::setgray}},
@@ -280,7 +286,10 @@ void PSInterpreter::callActions (InputReader &in) {
 		{"setlinewidth",   { 1, &PSActions::setlinewidth}},
 		{"setmatrix",      { 6, &PSActions::setmatrix}},
 		{"setmiterlimit",  { 1, &PSActions::setmiterlimit}},
+		{"setnulldevice",  { 1, &PSActions::setnulldevice}},
 		{"setopacityalpha",{ 1, &PSActions::setopacityalpha}},
+		{"setshapealpha",  { 1, &PSActions::setshapealpha}},
+		{"setpagedevice",  { 0, &PSActions::setpagedevice}},
 		{"setpattern",     {-1, &PSActions::setpattern}},
 		{"setrgbcolor",    { 3, &PSActions::setrgbcolor}},
 		{"shfill",         {-1, &PSActions::shfill}},
@@ -339,3 +348,32 @@ int GSDLLCALL PSInterpreter::error (void *inst, const char *buf, int len) {
 	return len;
 }
 
+
+/** Returns the total number of pages of a PDF file.
+ *  @param[in] fname name/path of the PDF file */
+int PSInterpreter::pdfPageCount (const string &fname) {
+	executeRaw("\n("+fname+")@pdfpagecount ", 1);
+	if (!_rawData.empty()) {
+		size_t index;
+		int ret = stoi(_rawData[0], &index, 10);
+		if (index > 0)
+			return ret;
+	}
+	return 0;
+}
+
+
+/** Returns the bounding box of a PDF page. If the selected page doesn't exist,
+ *  the "invalid" flag of the returned bounding box is set.
+ *  @param[in] fname name/path of the PDF file
+ *  @param[in] pageno page number
+ *  @return the bounding box of the given page */
+BoundingBox PSInterpreter::pdfPageBox (const string &fname, int pageno) {
+	BoundingBox pagebox;
+	executeRaw("\n"+to_string(pageno)+"("+fname+")@pdfpagebox ", 4);
+	if (_rawData.size() < 4)
+		pagebox.invalidate();
+	else
+		pagebox = BoundingBox(stod(_rawData[0]), stod(_rawData[1]), stod(_rawData[2]), stod(_rawData[3]));
+	return pagebox;
+}
