@@ -34,7 +34,7 @@
 
 
 class PSPattern;
-class XMLElementNode;
+class XMLElement;
 
 class PsSpecialHandler : public SpecialHandler, protected PSActions {
 	using Path = GraphicsPath<double>;
@@ -68,7 +68,7 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 				int saveID;        ///< if >=0, path was pushed by 'save', and saveID holds the ID of the PS memory object
 				Entry () : Entry(-1) {}
 				Entry (const Path &p, int pid, int sid) : path(std::make_shared<Path>(p)), pathID(pid), saveID(sid) {}
-				Entry (int sid) : path(nullptr), pathID(0), saveID(sid) {}
+				explicit Entry (int sid) : path(nullptr), pathID(0), saveID(sid) {}
 				Entry (const Entry &entry) =default;
 				Entry (Entry &&entry) =default;
 			};
@@ -77,11 +77,11 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 	};
 
 	enum PsSection {PS_NONE, PS_HEADERS, PS_BODY};
-	enum class FileType {EPS, PDF};
+	enum class FileType {EPS, PDF, SVG, BITMAP};
 
 	public:
 		PsSpecialHandler ();
-		~PsSpecialHandler ();
+		~PsSpecialHandler () override;
 		const char* name () const override {return "ps";}
 		const char* info () const override {return "dvips PostScript specials";}
 		std::vector<const char*> prefixes() const override;
@@ -103,7 +103,8 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 		void moveToDVIPos ();
 		void executeAndSync (std::istream &is, bool updatePos);
 		void processHeaderFile (const char *fname);
-		void imgfile (FileType type, const std::string &fname, const std::unordered_map<std::string,std::string> &attr);
+		void imgfile (FileType type, const std::string &fname, const std::map<std::string,std::string> &attr);
+		std::unique_ptr<XMLElement> createImageNode (FileType type, const std::string &fname, int pageno, BoundingBox bbox, bool clip);
 		void dviEndPage (unsigned pageno, SpecialActions &actions) override;
 		void clip (Path path, bool evenodd);
 		void processSequentialPatchMesh (int shadingTypeID, ColorSpace cspace, VectorIterator<double> &it);
@@ -145,6 +146,7 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 		void setlinewidth (std::vector<double> &p) override    {_linewidth = scale(p[0] ? p[0] : 0.5);}
 		void setmatrix (std::vector<double> &p) override;
 		void setmiterlimit (std::vector<double> &p) override   {_miterlimit = p[0];}
+		void setnulldevice (std::vector<double> &p) override;
 		void setopacityalpha (std::vector<double> &p) override {_opacityalpha = p[0];}
 		void setshapealpha (std::vector<double> &p) override   {_shapealpha = p[0];}
 		void setpagedevice (std::vector<double> &p) override;
@@ -160,8 +162,8 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 		SpecialActions *_actions;
 		PSPreviewFilter _previewFilter;  ///< filter to extract information generated by the preview package
 		PsSection _psSection=PS_NONE;    ///< current section processed (nothing yet, headers, or body specials)
-		XMLElementNode *_xmlnode;   ///< if != 0, created SVG elements are appended to this node
-		XMLElementNode *_savenode;  ///< pointer to temporaryly store _xmlnode
+		XMLElement *_xmlnode;   ///< if != 0, created SVG elements are appended to this node
+		XMLElement *_savenode;  ///< pointer to temporaryly store _xmlnode
 		std::string _headerCode;    ///< collected literal PS header code
 		Path _path;
 		DPair _currentpoint;        ///< current PS position in bp units
@@ -178,7 +180,7 @@ class PsSpecialHandler : public SpecialHandler, protected PSActions {
 		double _dashoffset;         ///< current dash offset
 		std::vector<double> _dashpattern;
 		ClippingStack _clipStack;
-		std::unordered_map<int, std::unique_ptr<PSPattern>> _patterns;
+		std::map<int, std::unique_ptr<PSPattern>> _patterns;
 		PSTilingPattern *_pattern;  ///< current pattern
 };
 
