@@ -1,8 +1,8 @@
 /*************************************************************************
 ** FilePathTest.cpp                                                     **
 **                                                                      **
-** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
+** This file is part of dvisvgm -- a fast DVI to SVG converter          **
+** Copyright (C) 2005-2019 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -20,12 +20,25 @@
 
 #include <gtest/gtest.h>
 #include <string>
-#include "FilePath.h"
-#include "FileSystem.h"
+#include "FilePath.hpp"
+#include "FileSystem.hpp"
+
+#ifndef SRCDIR
+#define SRCDIR "."
+#endif
 
 using namespace std;
 
 
+TEST(FilePathTest, empty) {
+	FilePath path;
+	ASSERT_TRUE(path.empty());
+	path.set("/a/b/c/d", false, "/");
+	ASSERT_FALSE(path.empty());
+	ASSERT_EQ(path.absolute(), "/a/b/c/d");
+}
+
+
 TEST(FilePathTest, dir1) {
 	FilePath fp("a/b/c/d", false, "/");
 	ASSERT_EQ(fp.absolute(), "/a/b/c/d");
@@ -74,34 +87,16 @@ TEST(FilePathTest, file2) {
 }
 
 
-#ifdef __WIN32__
-static string tolower (const string &str) {
-	string ret;
-	for (string::const_iterator it=str.begin(); it != str.end(); ++it)
-		ret += tolower(*it);
-	return ret;
-}
-#endif
-
-
 TEST(FilePathTest, autodetect) {
+	FileSystem::chdir(SRCDIR);
 	FilePath fp1("FilePathTest.cpp");
 	ASSERT_TRUE(fp1.isFile());
 	ASSERT_FALSE(fp1.empty());
 	string cwd = FileSystem::getcwd();
-#ifdef __WIN32__
-	ASSERT_EQ(fp1.absolute(), tolower(cwd + "/FilePathTest.cpp"));
-#else
-	ASSERT_EQ(fp1.absolute(), cwd + "/FilePathTest.cpp");
-#endif
+	ASSERT_EQ(fp1.absolute(), cwd + "/FilePathTest.cpp") << "fp1=" << fp1.absolute();
 
 	FilePath fp2("");
 	ASSERT_FALSE(fp2.isFile());
 	ASSERT_FALSE(fp2.empty());
-#ifdef __WIN32__
-	ASSERT_EQ(fp2.absolute(), tolower(FileSystem::getcwd()));
-#else
 	ASSERT_EQ(fp2.absolute(), FileSystem::getcwd());
-#endif
 }
-
