@@ -52,17 +52,17 @@ TEST(ShadingPatchTest, colorQueryFuncs) {
 	colors[1].setRGB(0.0, 1.0, 0.0);
 	colors[2].setRGB(0.0, 0.0, 1.0);
 	TriangularPatch tp1(points, colors, Color::ColorSpace::RGB, 0, 0);
-	EXPECT_EQ(tp1.colorAt(0, 0).rgbString(), "#ff0000");
-	EXPECT_EQ(tp1.colorAt(1, 0).rgbString(), "#00ff00");
-	EXPECT_EQ(tp1.colorAt(0, 1).rgbString(), "#0000ff");
+	EXPECT_EQ(tp1.colorAt(0, 0).rgbString(), "#f00");
+	EXPECT_EQ(tp1.colorAt(1, 0).rgbString(), "#0f0");
+	EXPECT_EQ(tp1.colorAt(0, 1).rgbString(), "#00f");
 
 	colors[0].setGray(0.2);
 	colors[1].setGray(0.4);
 	colors[2].setGray(0.6);
 	TriangularPatch tp2(points, colors, Color::ColorSpace::GRAY, 0, 0);
-	EXPECT_EQ(tp2.colorAt(0, 0).rgbString(), "#333333");
-	EXPECT_EQ(tp2.colorAt(1, 0).rgbString(), "#666666");
-	EXPECT_EQ(tp2.colorAt(0, 1).rgbString(), "#999999");
+	EXPECT_EQ(tp2.colorAt(0, 0).rgbString(), "#333");
+	EXPECT_EQ(tp2.colorAt(1, 0).rgbString(), "#666");
+	EXPECT_EQ(tp2.colorAt(0, 1).rgbString(), "#999");
 
 	colors[0].setCMYK(1.0, 0.0, 0.0, 0.2);
 	colors[1].setCMYK(0.0, 1.0, 0.0, 0.4);
