@@ -176,10 +176,10 @@ TEST_F(TensorProductPatchTest, vertices) {
 	EXPECT_EQ(_patch.valueAt(1,1), DPair(100,70));
 	EXPECT_EQ(_patch.valueAt(1,0), DPair(70,20));
 
-	EXPECT_EQ(_patch.colorAt(0,0).rgbString(), "#ff0000");
-	EXPECT_EQ(_patch.colorAt(0,1).rgbString(), "#ffff00");
-	EXPECT_EQ(_patch.colorAt(1,1).rgbString(), "#ff00ff");
-	EXPECT_EQ(_patch.colorAt(1,0).rgbString(), "#00ff00");
+	EXPECT_EQ(_patch.colorAt(0,0).rgbString(), "#f00");
+	EXPECT_EQ(_patch.colorAt(0,1).rgbString(), "#ff0");
+	EXPECT_EQ(_patch.colorAt(1,1).rgbString(), "#f0f");
+	EXPECT_EQ(_patch.colorAt(1,0).rgbString(), "#0f0");
 }
 
 
@@ -235,8 +235,7 @@ TEST_F(TensorProductPatchTest, values) {
 
 
 TEST_F(TensorProductPatchTest, boundaryPath) {
-	GraphicsPath<double> path;
-	_patch.getBoundaryPath(path);
+	GraphicsPath<double> path = _patch.getBoundaryPath();
 	ostringstream oss;
 	path.writeSVG(oss, false);
 	EXPECT_EQ(oss.str(), "M10 10C20 0 50 30 70 20C80 50 90 60 100 70C70 100 20 100 10 70C20 40 0 30 10 10Z");
@@ -246,12 +245,11 @@ TEST_F(TensorProductPatchTest, boundaryPath) {
 TEST_F(TensorProductPatchTest, subpatch) {
 	TensorProductPatch tpp;
 	_patch.subpatch(0, 0.5, 0, 0.5, tpp);
-	GraphicsPath<double> path;
-	tpp.getBoundaryPath(path);
+	GraphicsPath<double> path = tpp.getBoundaryPath();
 	ostringstream oss;
 	path.writeSVG(oss, false);
 	EXPECT_EQ(oss.str(), "M10 10C5 20 7.5 27.5 10 36.25C20.625 46.875 31.25 52.1875 43.28125 54.21875C40 40.9375 36.25 27.5 36.25 15C25 10 15 5 10 10Z");
-	EXPECT_EQ(tpp.colorAt(0, 0).rgbString(), "#ff0000");
+	EXPECT_EQ(tpp.colorAt(0, 0).rgbString(), "#f00");
 	EXPECT_EQ(tpp.colorAt(0, 1).rgbString(), "#ff8000");
 	EXPECT_EQ(tpp.colorAt(1, 0).rgbString(), "#808000");
 	EXPECT_EQ(tpp.colorAt(1, 1).rgbString(), "#bf8040");
@@ -259,8 +257,7 @@ TEST_F(TensorProductPatchTest, subpatch) {
 
 
 TEST_F(TensorProductPatchTest, bbox) {
-	BoundingBox bbox;
-	_patch.getBBox(bbox);
+	BoundingBox bbox = _patch.getBBox();
 	EXPECT_NEAR(bbox.minX(), 7.1132, 0.0001);
 	EXPECT_NEAR(bbox.minY(), 7.9289, 0.0001);
 	EXPECT_DOUBLE_EQ(bbox.maxX(), 100.0);
@@ -292,7 +289,7 @@ TEST_F(TensorProductPatchTest, approximate) {
 	TensorProductPatch tpp(_points, colors, Color::ColorSpace::RGB, 0, 0);
 	tpp.approximate(2, false, 0.1, callback);
 	EXPECT_EQ(callback.pathstr(), "M10 10C20 0 50 30 70 20C80 50 90 60 100 70C70 100 20 100 10 70C20 40 0 30 10 10Z");
-	EXPECT_EQ(callback.colorstr(), "#000000");
+	EXPECT_EQ(callback.colorstr(), "#000");
 
 	callback.reset();
 	_patch.approximate(2, false, 0.1, callback);
